@@ -27,7 +27,7 @@ import base64
 import json
 import os
 import requests
-import logging
+import logging 
 import traceback
 
 from datetime import datetime
@@ -39,12 +39,22 @@ try:
 except:
     pass 
 
-logging.basicConfig(format='%(asctime)s %(message)s', level=logging.DEBUG)
+logger = logging.getLogger('dataculpa')
+#logger.basicConfig(format='%(asctime)s %(message)s', level=logger.DEBUG)
+
+def log_error(s):
+    return
+
+def log_info(s):
+    return
+
+def log_debug(s):
+    return
 
 def show_versions():
     # print out some version debug stuff... 
     # see sklearn.show_versions() for inspiration
-    logging.error("Need to implement show_version!")
+    logger.error("Need to implement show_version!")
     return    
 
 class PipelineAlertConfig:
@@ -98,9 +108,11 @@ class DataCulpaValidator:
 
         self._open_queue()
 
+        self._pipeline_id = None
+
 
     def __del__(self):
-        logging.debug("DataCulpaValidator destructor called")
+        logger.debug("DataCulpaValidator destructor called")
 
 
     def test_connection(self):
@@ -108,21 +120,86 @@ class DataCulpaValidator:
         r = requests.get(url=url,
                          headers=self._json_headers())
         if r.status_code != 200:
-            logging.error("got status code %s for %s" % (r.status_code, url))
+            logger.error("got status code %s for %s" % (r.status_code, url))
             return 1
 
         try:
             jr = json.loads(r.content)
             if jr.get('status') is None:
-                logging.error("missing status")
+                logger.error("missing status")
                 return 1
         except:
-            logging.error("Error parsing result: __%s__", r.content)
+            logger.error("Error parsing result: __%s__", r.content)
             return 1
 
         # FIXME: needs more error handling.
         return 0
 
+    def _get_pipeline_id(self):
+        if self._pipeline_id is None:
+            pipe_id_request = self._get_base_url() + "data/metadata/pipeline-id/" + self._build_pipeline_url_suffix()
+            r = requests.get(url=pipe_id_request, headers=self._json_headers())
+            if r.status_code != 200:
+                raise Exception("unexpected result...status_code = ", r.status_code)
+
+            jr = None        
+            try:
+                jr = json.loads(r.content)
+                self._pipeline_id = jr.get('id')
+                print(pipe_id_request)
+                print(jr)
+            except:
+                logger.error("Error parsing result: __%s__", r.content)
+
+        return self._pipeline_id
+
+
+    def get_config(self):
+        _id = self._get_pipeline_id()
+
+        if _id is not None:
+            # now we can get the config...
+            config_url = self._get_base_url() + "data/metadata/pipeline-config/" + str(_id)
+            r = requests.get(url=config_url, headers=self._json_headers())
+            if r.status_code != 200:
+                raise Exception("unexpected result... perhaps the pipeline isn't defined yet?")
+
+            jr = None        
+            try:
+                jr = json.loads(r.content)
+            except:
+                logger.error("Error parsing result: __%s__", r.content)
+
+            return jr
+
+        return None
+
+    def get_recent_batchnames(self):
+        _id = self._get_pipeline_id()
+
+        if _id is not None:
+            # now we can get the config...
+            config_url = self._get_base_url() + "data/recent_batch_names/" + str(_id)
+            r = requests.get(url=config_url, headers=self._json_headers())
+            if r.status_code != 200:
+                raise Exception("unexpected result... perhaps the pipeline isn't defined yet?")
+
+            jr = None        
+            try:
+                jr = json.loads(r.content)
+            except:
+                logger.error("Error parsing result: __%s__", r.content)
+
+            return jr
+
+        return None
+
+    def set_use_gold(self, batch_entry):
+        pass
+
+    # add a new call for setting the gold standard configuration item...but to what?
+    # we need the list of potential files we could use.
+
     def _get_base_url(self):
         return "%s://%s:%s/" % (self.protocol, self.host, self.port)
 
@@ -179,14 +256,14 @@ class DataCulpaValidator:
 
             return True # worked.
         except requests.exceptions.Timeout:
-            logging.error("timed out trying to load csv file...")
+            logger.error("timed out trying to load csv file...")
 #        except requests.exceptions.RetryError:
         except requests.exceptions.HTTPError as err:
-            logging.error("got an http error: %s" % err)
+            logger.error("got an http error: %s" % err)
         except requests.RequestException as e:
-            logging.error("got an request error... server down?: %s" % e)
-        except:
-            logging.error("got some other error:")
+            logger.error("got an request error... server down?: %s" % e)
+        except BaseException as e:
+            logger.error("got some other error: %s" % e)
  
         return False # got an error.
 
@@ -206,7 +283,9 @@ class DataCulpaValidator:
         assert isinstance(meta, dict), "meta must be a dict"
         
         queue_id = None
-        self._open_queue()
+        if self._queue_id is None:
+            self._open_queue()
+
         if self._queue_id is not None:
             queue_id = self._queue_id
         assert queue_id is not None, "open queue failed" # FIXME: handle this better.
@@ -239,6 +318,13 @@ class DataCulpaValidator:
     def _append_error(self, message):
         self._queue_errors.append( { 'when': datetime.utcnow(), 'message': message })
         return
+    
+    def has_errors(self):
+        # FIXME: we're very inconsistent on how we deal with errors right now.
+        return self._queue_errors != []
+    
+    def get_errors(self):
+        return self._queue_errors
 
     def _calc_timeshift_seconds(self):
         ts = self._timeshift
@@ -246,7 +332,7 @@ class DataCulpaValidator:
             return 0
         
         # if it's an int, pass that
-        if type(ts) == int:
+        if type(ts) == int or type(ts) == float:
             return ts
 
         # if it's a datetime, calculate the delta to now in seconds.
@@ -270,7 +356,7 @@ class DataCulpaValidator:
     def _open_queue(self):
         j = { 
                 'pipeline' : self.pipeline_name,
-                'context ' : self.pipeline_environment,
+                'context'  : self.pipeline_environment,
                 'stage'    : self.pipeline_stage,
                 'version'  : self.pipeline_version,
                 'timeshift': self._calc_timeshift_seconds()
@@ -289,7 +375,7 @@ class DataCulpaValidator:
             self._queue_buffer = []
         except:
             # FIXME: need to handle ConnectionRefusedError and so on!
-            logging.info("Probably got a time out...") 
+            logger.info("Probably got a time out...") 
             traceback.print_exc()
             return
 
@@ -297,9 +383,8 @@ class DataCulpaValidator:
             try:
                 jr = json.loads(r.content)
                 self._queue_id = jr.get('queue_id')
-                print("queue_id = ", self._queue_id)
             except:
-                logging.debug("Error parsing result: __%s__", r.content)
+                logger.debug("Error parsing result: __%s__", r.content)
 
         return # (None, 0, 0)
 
@@ -311,7 +396,7 @@ class DataCulpaValidator:
 
         rs_str = json.dumps(self._queue_buffer, cls=self._jsonEncoder, default=str)
         post_url = self._get_base_url() + path
-        #logging.debug("%s: about to post %d bytes to %s" % (datetime.now(), len(rs_str), post_url))
+        #logger.debug("%s: about to post %d bytes to %s" % (datetime.now(), len(rs_str), post_url))
 
         r = None
 
@@ -323,7 +408,7 @@ class DataCulpaValidator:
             self._queue_buffer = []
         except:
             # FIXME: need to handle ConnectionRefusedError and so on!
-            logging.info("Probably got a time out...") 
+            logger.info("Probably got a time out...") 
             traceback.print_exc()
 
         if r is not None:
@@ -333,7 +418,7 @@ class DataCulpaValidator:
                 #return jr.get('queue_id'), jr.get('queue_count'), jr.get('queue_age')
                 # FIXME: improve error handling.
             except:
-                logging.debug("Error parsing result: __%s__", r.content)
+                logger.debug("Error parsing result: __%s__", r.content)
 
         return # (None, 0, 0)
 
@@ -382,5 +467,52 @@ class DataCulpaValidator:
             jr = json.loads(r.content)
             return jr
         except:
-            logging.error("Error parsing result: __%s__", r.content)
+            logger.error("Error parsing result: __%s__", r.content)
         return None
+
+#    def debug_list_pipelines():
+#
+
+    def query_alerts(self):
+        # get alerts for this pipeline
+        pid = self._get_pipeline_id()
+        assert pid is not None, "FIXME: return an error"
+        _url = self._get_base_url() + "data/metadata/alerts/%s" % pid
+
+        r = requests.get(url=_url, headers=self._json_headers())
+        if r.status_code != 200:
+            raise Exception("unexpected result... perhaps the pipeline isn't defined yet?")
+        
+        jr = None        
+        try:
+            jr = json.loads(r.content)
+        except:
+            logger.error("Error parsing result: __%s__", r.content)
+
+        return jr
+
+
+    def query_get_raw_data(self):
+        # fetch the recent data for this pipeline, up to ~100 recent records.
+        # this is intended for verification of operations and may not perform well on large data sets.
+        return
+
+    def query_summary_data(self):
+        # get the summary for this pipeline
+        pid = self._get_pipeline_id()
+        assert pid is not None, "FIXME: return an error"
+
+        _url = self._get_base_url() + "data/aggr/%s" % pid
+        r = requests.get(url=_url, headers=self._json_headers())
+        if r.status_code != 200:
+            raise Exception("unexpected result... perhaps the pipeline id = %s isn't defined yet?" % pid)
+
+        jr = None        
+        try:
+            jr = json.loads(r.content)
+        except:
+            logger.error("Error parsing result: __%s__", r.content)
+
+        return jr
+
+    
